# coding=utf-8
from __future__ import unicode_literals, print_function, absolute_import

import asyncio
__title__ = 'bw'
__description__ = '牛富量化 python3 x64 sdk'
__url__ = 'http://bwsoft.cn/'
__version__ = '1.5.86'
__author__ = 'fengcl'
__author_email__ = '439126089@qq.com'
__license__ = 'Apache 2.0'
